@@ -30,7 +30,7 @@ from urllib import unquote
 
 from imdb.Person import Person
 from imdb.Movie import Movie
-from imdb.utils import analyze_title
+from imdb.utils import analyze_title, re_episodes
 from imdb._exceptions import IMDbParserError
 from utils import ParserBase
 
@@ -77,6 +77,9 @@ class HTMLMovieParser(ParserBase):
 
     def _init(self):
         self.__movie_data = {}
+        # If true, we're parsing the "maindetails" page; if false,
+        # the "combined" page is expected.
+        self.mdparse = 0
 
     def _reset(self):
         self.__movie_data.clear()
@@ -112,6 +115,9 @@ class HTMLMovieParser(ParserBase):
         self.__runtimes = ''
         self.__is_mpaa = 0
         self.__mpaa = ''
+        self.__inbch = 0
+        self.__isplotoutline = 0
+        self.__plotoutline = ''
         # If true, the next data should be merged with the previous one,
         # without the '::' separator.
         self.__merge_next = 0
@@ -202,6 +208,10 @@ class HTMLMovieParser(ParserBase):
             self.__is_languages = 1
         elif link.startswith('/mpaa'):
             self.__is_mpaa = 1
+        elif self.__isplotoutline:
+            self.__isplotoutline = 0
+            if self.__plotoutline:
+                self.set_item('plot outline', self.__plotoutline)
         elif link.startswith('http://pro.imdb.com'):
             self.__is_movie_status = 0
         elif link.startswith('/titlebrowse?'):
@@ -226,15 +236,20 @@ class HTMLMovieParser(ParserBase):
                 n_split = self.__name.split('::')
                 n = n_split[0].strip()
                 del n_split[0]
-                role = ' '.join(n_split)
+                role = ' '.join(n_split).strip()
                 notes = ''
                 ii = role.find('(')
                 if ii != -1:
                     ei = role.rfind(')')
                     if ei != -1:
-                        notes = role[ii:ei+1]
+                        notes = role[ii:ei+1].strip()
                         role = '%s%s' % (role[:ii], role[ei+1:])
-                        role = role.replace('  ', ' ')
+                        role = role.replace('  ', ' ').strip()
+                sect = clear_text(self.__current_section)
+                if sect != 'cast':
+                    if notes: notes = ' %s' % notes
+                    notes = role + notes
+                    role = ''
                 # Create a Person object.
                 # XXX: check for self.__cur_nameID?
                 #      maybe it's not a good idea; it's possible for
@@ -242,7 +257,6 @@ class HTMLMovieParser(ParserBase):
                 p = Person(name=n, currentRole=role,
                             personID=self.__cur_nameID, accessSystem='http')
                 if notes: p.notes = notes
-                sect = clear_text(self.__current_section)
                 if not self.__movie_data.has_key(sect):
                     self.__movie_data[sect] = []
                 self.__movie_data[sect].append(p)
@@ -261,6 +275,9 @@ class HTMLMovieParser(ParserBase):
                 if not sect_name.startswith('status'):
                     sect_name = 'status %s' % sect_name
                 self.set_item(sect_name, self.__movie_status_data)
+        elif self.__is_cast_crew and self.__current_section:
+            if self.__is_name:
+                self.__name += '::'
     
     def do_br(self, attrs):
         if self.__is_company_cred:
@@ -302,18 +319,22 @@ class HTMLMovieParser(ParserBase):
             self.__is_mpaa = 0
             mpaa = self.__mpaa.replace('MPAA:', '')
             self.set_item('mpaa', mpaa)
+        elif self.__isplotoutline:
+            self.__isplotoutline = 0
+            if self.__plotoutline:
+                self.set_item('plot outline', self.__plotoutline)
         elif self.__is_runtimes and self.__runtimes:
             self.__is_runtimes = 0
             rt = self.__runtimes.replace(' min', '')
             # The "(xy episodes)" note.
-            sep = rt.find(' (')
-            if sep != -1:
-                eep = rt.find(')')
-                if eep > sep:
-                    self.set_item('notes', rt[sep:eep+1])
-                    rt = '%s%s' % (rt[:sep], rt[eep+1:])
+            episodes = re_episodes.findall(rt)
+            if episodes:
+                rt = re_episodes.sub('', rt)
+                self.set_item('episodes', episodes[0])
             rl = [x.strip() for x in rt.split('/')]
             if rl: self.set_item('runtimes', rl)
+        if self.mdparse:
+            self.end_tr()
 
     def start_li(self, attrs): pass
 
@@ -332,8 +353,16 @@ class HTMLMovieParser(ParserBase):
 
     def start_b(self, attrs):
         self.__is_akas = 0
+        cls = self.get_attr_value(attrs, 'class')
+        if cls:
+            cls = cls.lower()
+            if cls == 'ch':
+                self.__inbch = 1
+            elif self.mdparse and cls == 'blackcatheader':
+                self.end_table()
 
-    def end_b(self): pass
+    def end_b(self):
+        if self.__inbch: self.__inbch = 0
 
     def do_img(self, attrs):
         alttex = self.get_attr_value(attrs, 'alt')
@@ -356,7 +385,7 @@ class HTMLMovieParser(ParserBase):
                 if i != -1:
                     rating = rav[:i]
                     try:
-                        float(rating)
+                        rating = float(rating)
                         self.set_item('rating', rating)
                     except ValueError:
                         pass
@@ -366,7 +395,7 @@ class HTMLMovieParser(ParserBase):
                     j = votes.find(' ')
                     votes = votes[:j].replace(',', '')
                     try:
-                        int(votes)
+                        votes = int(votes)
                         self.set_item('votes', votes)
                     except ValueError:
                         pass
@@ -470,12 +499,28 @@ class HTMLMovieParser(ParserBase):
         elif self.__is_languages:
             self.append_item('languages', data)
             self.__is_languages = 0
+        elif self.__isplotoutline:
+            self.__plotoutline += data
+        elif self.__inbch and sldata.startswith('plot outline:'):
+            self.__isplotoutline = 1
         elif sldata.startswith('also known as'):
             self.__is_akas = 1
-        elif sldata.startswith('production notes/status'):
-            self.__is_movie_status = 1
         elif sldata.startswith('runtime:'):
             self.__is_runtimes = 1
+        elif sldata.startswith('production notes/status'):
+            self.__is_movie_status = 1
+        # XXX: the following branches are here to manage the "maindetails"
+        #      page of a movie, instead of the "combined" page.
+        if self.mdparse:
+            if sldata.startswith('cast overview, first billed only'):
+                self.__is_cast_crew = 1
+                self.__current_section = 'cast'
+            elif sldata.startswith('directed by'):
+                self.__is_cast_crew = 1
+                self.__current_section = 'director'
+            elif sldata.startswith('writing credits'):
+                self.__is_cast_crew = 1
+                self.__current_section = 'writer'
 
 
 class HTMLPlotParser(ParserBase):
@@ -916,9 +961,18 @@ class HTMLAlternateVersionsParser(ParserBase):
         self.__in_avd = 0
         self.__av = []
         self.__cav = ''
+        self.__stlist = []
+        self.__curst = {}
+        self.__cur_title = ''
+        self.__curinfo = ''
 
     def get_data(self):
         """Return the dictionary."""
+        if self.kind == 'soundtrack':
+            if self.__stlist:
+                return {self.kind: self.__stlist}
+            else:
+                return {}
         if not self.__av: return {}
         return {self.kind: self.__av}
 
@@ -935,10 +989,49 @@ class HTMLAlternateVersionsParser(ParserBase):
 
     def end_li(self):
         if self.__in_av and self.__in_avd:
+            if self.kind == 'soundtrack':
+                self.__stlist.append(self.__curst.copy())
+                self.__curst.clear()
+                self.__cur_title = ''
+                self.__curinfo = ''
+            else:
+                self.__av.append(self.__cav.strip())
             self.__in_avd = 0
-            self.__av.append(self.__cav.strip())
             self.__cav = ''
 
+    def do_br(self, attrs):
+        if self.__in_avd and self.kind == 'soundtrack':
+            if not self.__cur_title:
+                self.__cav = self.__cav.strip()
+                if self.__cav and self.__cav[-1] == '"':
+                    self.__cav = self.__cav[:-1]
+                if self.__cav and self.__cav[0] == '"':
+                    self.__cav = self.__cav[1:]
+                self.__cur_title = self.__cav
+                self.__curst[self.__cur_title] = {}
+                self.__cav = ''
+            else:
+                lcw = self.__cav.lower()
+                for i in ('with', 'by', 'from', 'of'):
+                    posi = lcw.find(i)
+                    if posi != -1:
+                        self.__curinfo = self.__cav[:posi+len(i)]
+                        if self.kind == 'soundtrack':
+                            self.__curinfo = self.__curinfo.lower().strip()
+                        rest = self.__cav[posi+len(i)+1:]
+                        self.__curst[self.__cur_title][self.__curinfo] = \
+                                rest
+                        break
+                else:
+                    if not lcw.strip(): return
+                    if not self.__curst[self.__cur_title].has_key('misc'):
+                        self.__curst[self.__cur_title]['misc'] = ''
+                    if self.__curst[self.__cur_title]['misc'] and \
+                            self.__curst[self.__cur_title]['misc'][-1] != ' ':
+                        self.__curst[self.__cur_title]['misc'] += ' '
+                    self.__curst[self.__cur_title]['misc'] += self.__cav
+                self.__cav = ''
+
     def _handle_data(self, data):
         if self.__in_avd:
             self.__cav += data
@@ -1223,6 +1316,7 @@ class HTMLRatingsParser(ParserBase):
         self.__cur_demo_av = ''
         self.__next_is_demo_vote = 0
         self.__next_demo_vote = ''
+        self.__in_td = 0
 
     def get_data(self):
         """Return the dictionary."""
@@ -1247,9 +1341,11 @@ class HTMLRatingsParser(ParserBase):
     def end_b(self):
         self.__in_b = 0
 
-    def start_td(self, attrs): pass
+    def start_td(self, attrs):
+        self.__in_td = 1
 
     def end_td(self):
+        self.__in_td = 0
         if self.__in_total:
             if self.__first:
                 self.__first = 0
@@ -1262,10 +1358,8 @@ class HTMLRatingsParser(ParserBase):
         if self.__in_total:
             if self.__cur_nr:
                 try:
-                    c = self.__cur_vote
-                    n = self.__cur_nr
-                    int(c)
-                    int(n)
+                    c = int(self.__cur_vote)
+                    n = int(self.__cur_nr)
                     self.__votes[c] = n
                 except (ValueError, OverflowError): pass
                 self.__cur_nr = ''
@@ -1273,10 +1367,8 @@ class HTMLRatingsParser(ParserBase):
         if self.__in_demo:
             self.__in_demo = 0
             try:
-                av = self.__cur_demo_av
-                dv = self.__next_demo_vote
-                float(av)
-                int(dv)
+                av = float(self.__cur_demo_av)
+                dv = int(self.__next_demo_vote)
                 self.__demo[self.__cur_demo_t] = (dv, av)
             except (ValueError, OverflowError): pass
             self.__cur_demo_av = ''
@@ -1314,8 +1406,22 @@ class HTMLRatingsParser(ParserBase):
                 if i != -1:
                     sd = sd[:i]
                     try: sd = int(sd)
-                    except (ValueError, OverflowError): return
-                    self.__rank = {'top 250 rank': sd}
+                    except (ValueError, OverflowError): pass
+                    if type(sd) is type(1):
+                        self.__rank['top 250 rank'] = sd
+            elif sdata.startswith('Arithmetic mean = '):
+                if sdata[-1] == '.': sdata = sdata[:-1]
+                am = sdata[18:]
+                try: am = float(am)
+                except (ValueError, OverflowError): pass
+                if type(am) is type(1.0):
+                    self.__rank['arithmetic mean'] = am
+            elif sdata.startswith('Median = '):
+                med = sdata[9:]
+                try: med = int(med)
+                except (ValueError, OverflowError): pass
+                if type(med) is type(1):
+                    self.__rank['median'] = med
         if self.__in_demo:
             if self.__next_is_demo_vote:
                 self.__next_demo_vote = sdata
@@ -1325,6 +1431,10 @@ class HTMLRatingsParser(ParserBase):
                 self.__next_is_demo_vote = 1
             else:
                 self.__cur_demo_av = sdata
+        elif self.__in_td and sdata.startswith('All votes'):
+            self.__in_demo = 1
+            self.__next_is_demo_vote = 1
+            self.__cur_demo_t = 'all votes'
 
 
 class HTMLOfficialsitesParser(ParserBase):
@@ -1465,21 +1575,458 @@ class HTMLConnectionParser(ParserBase):
             self.__cnt += data.lower()
 
 
+class HTMLTechParser(ParserBase):
+    """Parser for the "technical", "business", "literature",
+    "publicity" (for people) and "locations" pages of a given movie.
+    The page should be provided as a string, as taken from
+    the akas.imdb.com server.  The final result will be a
+    dictionary, with a key for every relevant section.
+
+    Example:
+        tparser = HTMLTechParser()
+        result = tparser.parse(technical_html_string)
+    """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
+    def _init(self):
+        self.kind = 'something else'
+    
+    def _reset(self):
+        """Reset the parser."""
+        self.__tc = {}
+        self.__dotc = 0
+        self.__indt = 0
+        self.__indd = 0
+        self.__cur_sect = ''
+        self.__curdata = ['']
+    
+    def get_data(self):
+        """Return the dictionary."""
+        if self.kind == 'locations':
+            rl = []
+            for item in self.__tc.items():
+                tmps = item[0].strip() + ' ' + \
+                        ' '.join([x.strip() for x in item[1]])
+                rl.append(tmps)
+            if rl: return {'locations': rl}
+        return self.__tc
+
+    def start_dl(self, attrs):
+        self.__dotc = 1
+
+    def end_dl(self):
+        self.__dotc = 0
+
+    def start_dt(self, attrs):
+        if self.__dotc: self.__indt = 1
+
+    def end_dt(self):
+        self.__indt = 0
+
+    def start_tr(self, attrs): pass
+
+    def end_tr(self):
+        if self.__indd and self.kind == 'publicity':
+            if self.__curdata:
+                self.do_br([])
+
+    def start_td(self, attrs): pass
+
+    def end_td(self):
+        if self.__indd and self.__curdata and self.kind == 'publicity':
+            if self.__curdata[-1].find('::') == -1:
+                self.__curdata[-1] += '::'
+
+    def start_p(self, attrs): pass
+
+    def end_p(self):
+        if self.__indd and self.kind == 'publicity':
+            if self.__curdata:
+                self.__curdata[-1] += '::'
+                self.do_br([])
+
+    def start_dd(self, attrs):
+        if self.__dotc: self.__indd = 1
+
+    def end_dd(self):
+        self.__indd = 0
+        self.__curdata[:] = [x.strip() for x in self.__curdata]
+        self.__curdata[:] = [x for x in self.__curdata if x]
+        for i in xrange(len(self.__curdata)):
+            if self.__curdata[i][-2:] == '::':
+                self.__curdata[i] = self.__curdata[i][:-2]
+        if self.__cur_sect and self.__curdata:
+            self.__tc[self.__cur_sect] = self.__curdata[:]
+        self.__curdata[:] = ['']
+        self.__cur_sect = ''
+
+    def do_br(self, attrs):
+        if self.__indd:
+            self.__curdata += ['']
+
+    def _handle_data(self, data):
+        if self.__indd:
+            self.__curdata[-1] += data
+        elif self.__indt:
+            if self.kind != 'locations': data = data.lower()
+            self.__cur_sect += data
+
+
+class HTMLDvdParser(ParserBase):
+    """Parser for the "dvd" page of a given movie.
+    The page should be provided as a string, as taken from
+    the akas.imdb.com server.  The final result will be a
+    dictionary, with a key for every relevant section.
+
+    Example:
+        dparser = HTMLDvdParser()
+        result = dparser.parse(dvd_html_string)
+    """
+    # TODO: it's not still ready to handle the "laserdisc" page.
+    kind = 'dvd'
+
+    def _init(self):
+        self.__dvd = []
+
+    def _reset(self):
+        """Reset the parser."""
+        self.__cdvd = {}
+        self.__indvd = 0
+        self.__intitle = 0
+        self.__curtitle = ''
+        self.__binfo = 0
+        self.__binfo_txt = ''
+        self.__inth = 0
+        self.__noendb = 0
+        self.__finfo = 0
+        self.__finfo_txt = ''
+        self.__inar = 0
+        self.__thl = []
+        self.__thmult = 1
+        self.__colcount = -1
+        self.__coldata = ''
+        self.__insound = 0
+
+    def get_data(self):
+        """Return the dictionary."""
+        if self.__dvd: return {'dvd': self.__dvd}
+        return {}
+
+    def _put_data(self, s):
+        s = s.strip()
+        if not s: return
+        dcmi = s.find('::-')
+        mini = s.find('min')
+        if dcmi != -1 and mini != -1:
+            self._put_data('color::%s' % s[:dcmi])
+            self._put_data('runtime::%s' % s[dcmi+3:mini])
+            return
+        ss = [x.strip() for x in s.split('::') if x.strip()]
+        ssl = len(ss)
+        if ssl == 1:
+            if ss[0].lower().find('read about how we rate dvds') != -1: return
+            if not self.__cdvd.has_key('misc'):
+                self.__cdvd['misc'] = []
+            self.__cdvd['misc'].append(ss[0])
+        else:
+            k = ss[0].lower()
+            v = ' '.join(ss[1:]).replace('\n', '')
+            if self.__cdvd.has_key(k) and type(self.__cdvd[k]) is not type([]):
+                self.__cdvd[k] = [self.__cdvd[k]]
+            if self.__cdvd.has_key(k):
+                self.__cdvd[k] += v
+            else:
+                self.__cdvd[k] = v
+
+    def do_hr(self, attrs):
+        if self.__indvd and self.__cdvd:
+            self.__dvd.append(self.__cdvd.copy())
+            self._reset()
+
+    def start_a(self, attrs):
+        name = self.get_attr_value(attrs, 'name')
+        if name and len(name) > 1 and name[0] == "X":
+            self.__indvd = 1
+        if not self.__indvd: return
+        href = self.get_attr_value(attrs, 'href')
+        hrefl = href
+        if href: hrefl = href.lower()
+        if hrefl and hrefl.find('sections/dvds/labels') != -1:
+            self.__binfo_txt = 'label::'
+            self.__noendb = 1
+        elif hrefl and hrefl.find('sections/dvds/pictureformats') != -1:
+            self.__binfo_txt = 'picture format::'
+            self.__noendb = 1
+        elif hrefl and hrefl.find('sections/dvds/regions') != -1:
+            self.__finfo_txt = 'region::'
+        elif hrefl and hrefl.find('sections/dvds/video') != -1:
+            self.__finfo_txt = 'video standard::'
+        elif hrefl and hrefl.find('sections/dvds/packaging') != -1:
+            self.__finfo_txt = 'packaging::'
+
+    def end_a(self): pass
+
+    def do_br(self, attrs):
+        if self.__indvd:
+            if self.__intitle: self.__curtitle += ' - '
+            if self.__binfo:
+                self.__binfo = 0
+                if self.__binfo_txt:
+                    self._put_data(self.__binfo_txt)
+                    self.__binfo_txt = ''
+
+    def start_table(self, attrs):
+        self.__thl = []
+
+    def end_table(self):
+        if self.__insound: self.__insound = 0
+
+    def start_th(self, attrs):
+        if self.__indvd:
+            self.__inth = 1
+            span = self.get_attr_value(attrs, 'colspan')
+            if span:
+                try:self.__thmult = int(span)
+                except (ValueError, OverflowError): pass
+
+    def end_th(self):
+        self.__inth = 0
+
+    def start_tr(self, attrs):
+        if not self.__indvd: return
+        if self.__thl:
+            self.__colcount = -1
+
+    def end_tr(self): pass
+
+    def start_td(self, attrs):
+        if not self.__indvd: return
+        if self.__thl:
+            self.__colcount += 1
+            self.__colcount %= len(self.__thl)
+
+    def end_td(self):
+        if not self.__indvd: return
+        if self.__thl:
+            self.__coldata = self.__coldata.strip()
+            if not self.__coldata: return
+            if not self.__insound:
+                k = self.__thl[self.__colcount]
+                v = self.__coldata.strip().replace('\n', ' ').replace('  ', ' ')
+                if v.find('::') != -1:
+                    v = [x.strip() for x in v.split('::') if x.strip()]
+                if not (k and v): return
+                self.__cdvd[k] = v
+            else:
+                if not self.__cdvd.has_key('sound'): self.__cdvd['sound'] = {}
+                lang = self.__thl[self.__colcount].lower()
+                self.__cdvd['sound'][lang] = self.__coldata
+            self.__coldata = ''
+
+    def start_li(self, attrs):
+        if self.__indvd and self.__thl and self.__coldata.strip():
+            self.__coldata += '::'
+
+    def end_li(self): pass
+            
+    def start_h3(self, attrs):
+        if self.__indvd:
+            self.__intitle = 1
+
+    def end_h3(self):
+        if self.__indvd:
+            self.__intitle = 0
+            self.__cdvd['title'] = self.__curtitle
+
+    def start_font(self, attrs):
+        if not self.__indvd: return
+        cls = self.get_attr_value(attrs, 'class')
+        if cls and cls.lower() in ('catheader', 'smalltxt') and \
+                not self.__inth and not self.__binfo:
+            self.__finfo = 1
+
+    def end_font(self):
+        if not self.__indvd: return
+        if self.__finfo:
+            self.__finfo = 0
+            self.__finfo_txt = self.__finfo_txt.strip()
+            if self.__finfo_txt:
+                self._put_data(self.__finfo_txt)
+                self.__finfo_txt = ''
+
+    def do_img(self, attrs):
+        src = self.get_attr_value(attrs, 'src')
+        if src and src.lower().find('images.amazon.com') != -1:
+            self.__cdvd['cover'] = src
+
+    def start_b(self, attrs):
+        if self.kind == 'laserdisc':
+            cls = self.get_attr_value(attrs, 'class')
+            if cls and cls.lower() == 'ch':
+                if not self.__indvd:
+                    self.__indvd = 1
+        if not self.__indvd: return
+        cls = self.get_attr_value(attrs, 'class')
+        if cls and cls.lower() in ('catheader', 'smalltxt', 'aspectratio') \
+                and not self.__inth:
+            self.__binfo = 1
+            self.__binfo_txt = ''
+            if cls.lower() == 'aspectratio': self.__inar = 1
+
+    def end_b(self):
+        if self.__binfo:
+            self.__binfo_txt = self.__binfo_txt.strip()
+            if self.__binfo_txt:
+                if self.__binfo_txt[-1] == ':':
+                    self.__binfo_txt = self.__binfo_txt[:-1]
+                sep = '::'
+                if self.__noendb:
+                    sep = ' '
+                    self.__noendb = 0
+                self.__binfo_txt += sep
+            if  self.__inar:
+                self.__inar = 0
+                self.__binfo_txt = self.__binfo_txt.replace(' : ', ':')
+                self._put_data('aspect ratio::%s' % self.__binfo_txt)
+
+    def _handle_data(self, data):
+        if self.__indvd:
+            if self.__inth:
+                dsl = data.replace(':', '').strip().lower()
+                self.__thl += [dsl]*self.__thmult
+                if dsl == 'sound':
+                    self.__insound = 1
+            if self.__intitle:
+                self.__curtitle += data
+            elif self.__binfo:
+                sdata = data.lstrip()
+                if sdata:
+                    self.__binfo_txt += sdata
+            elif self.__finfo:
+                sdata = data.lstrip()
+                if sdata:
+                    self.__finfo_txt += sdata
+            elif self.__thl and not self.__inth:
+                self.__coldata += data
+
+
+
+class HTMLRecParser(ParserBase):
+    """Parser for the "recommendations" page of a given movie.
+    The page should be provided as a string, as taken from
+    the akas.imdb.com server.  The final result will be a
+    dictionary, with a key for every relevant section.
+
+    Example:
+        rparser = HTMLRecParser()
+        result = rparser.parse(recommendations_html_string)
+    """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
+    def _reset(self):
+        """Reset the parser."""
+        self.__rec = {}
+        self.__firsttd = 0
+        self.__curlist = ''
+        self.__curtitle = ''
+        self.__startgath = 0
+        self.__intable = 0
+        self.__inb = 0
+        self.__cur_id = ''
+    
+    def get_data(self):
+        if not self.__rec: return {}
+        return {'recommendations': self.__rec}
+
+    def start_a(self, attrs):
+        if self.__firsttd:
+            href = self.get_attr_value(attrs, 'href')
+            if href:
+                tn = self.re_imdbID.findall(href)
+                if tn:
+                    self.__cur_id = tn[-1]
+
+    def end_a(self): pass
+
+    def start_table(self, attrs):
+        self.__intable = 1
+
+    def end_table(self):
+        self.__intable = 0
+        self.__startgath = 0
+
+    def start_tr(self, attrs):
+        self.__firsttd = 1
+
+    def end_tr(self): pass
+
+    def start_td(self, attrs):
+        if self.__firsttd:
+            span = self.get_attr_value(attrs, 'colspan')
+            if span: self.__firsttd = 0
+
+    def end_td(self):
+        if self.__firsttd:
+            self.__curtitle = clear_text(self.__curtitle)
+            if self.__curtitle:
+                if self.__curlist:
+                    if not self.__rec.has_key(self.__curlist):
+                        self.__rec[self.__curlist] = []
+                    if self.__cur_id:
+                        m = Movie(movieID=self.__cur_id,
+                                    movieData=analyze_title(self.__curtitle,
+                                                            canonical=1),
+                                    accessSystem='http')
+                        self.__rec[self.__curlist].append(m)
+                        self.__cur_id = ''
+                self.__curtitle = ''
+            self.__firsttd = 0
+
+    def start_b(self, attrs):
+        self.__inb = 1
+
+    def end_b(self):
+        self.__inb = 0
+
+    def _handle_data(self, data):
+        ldata = data.lower()
+        if self.__intable and self.__inb:
+            if ldata.find('suggested by the database') != -1:
+                self.__startgath = 1
+                self.__curlist = 'database'
+            elif ldata.find('imdb users recommend') != -1:
+                self.__startgath = 1
+                self.__curlist = 'users'
+        elif self.__firsttd and self.__curlist:
+            self.__curtitle += data
+
+
 # The used instances.
 movie_parser = HTMLMovieParser()
 plot_parser = HTMLPlotParser()
 movie_awards_parser = HTMLAwardsParser()
 taglines_parser = HTMLTaglinesParser()
 keywords_parser = HTMLKeywordsParser()
-alternateversions_parser = HTMLAlternateVersionsParser()
 crazycredits_parser = HTMLCrazyCreditsParser()
 goofs_parser = HTMLGoofsParser()
+alternateversions_parser = HTMLAlternateVersionsParser()
 trivia_parser = HTMLAlternateVersionsParser()
+soundtrack_parser = HTMLAlternateVersionsParser()
 trivia_parser.kind = 'trivia'
+soundtrack_parser.kind = 'soundtrack'
 quotes_parser = HTMLQuotesParser()
 releasedates_parser = HTMLReleaseinfoParser()
 ratings_parser = HTMLRatingsParser()
 officialsites_parser = HTMLOfficialsitesParser()
 connections_parser = HTMLConnectionParser()
+tech_parser = HTMLTechParser()
+locations_parser = HTMLTechParser()
+locations_parser.kind = 'locations'
+dvd_parser = HTMLDvdParser()
+rec_parser = HTMLRecParser()
 
 
