@@ -4,9 +4,9 @@ parser package (imdb package).
 This package provides various parser to access IMDb data (e.g.: a
 parser for the web/http interface, a parser for the local files,
 a parser for the e-mail interface, etc.).
-So far, only the http parser is implemented.
+So far, the http and local parsers are implemented.
 
-Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>
+Copyright 2004, 2005 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,6 +23,6 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-__all__ = ['http']
+__all__ = ['http', 'local', 'mobile']
 
 
