@@ -33,11 +33,14 @@ from movieParser import movie_parser, plot_parser, movie_awards_parser, \
                         alternateversions_parser, crazycredits_parser, \
                         goofs_parser, trivia_parser, quotes_parser, \
                         releasedates_parser, ratings_parser, \
-                        officialsites_parser, connections_parser
+                        officialsites_parser, connections_parser, \
+                        tech_parser, locations_parser, soundtrack_parser, \
+                        dvd_parser, rec_parser
 from searchMovieParser import search_movie_parser
 from personParser import maindetails_parser, bio_parser, \
                         otherworks_parser, person_awards_parser, \
-                        person_officialsites_parser
+                        person_officialsites_parser, publicity_parser, \
+                        agent_parser
 from searchPersonParser import search_person_parser
 
 
@@ -46,6 +49,11 @@ imdbURL_movie = 'http://akas.imdb.com/title/tt%s/'
 imdbURL_person = 'http://akas.imdb.com/name/nm%s/'
 imdbURL_search = 'http://akas.imdb.com/find?%s'
 
+# The cookies for the "adult" search.
+# Please don't mess with this account.
+_cookie_id = 'boM2bYxz9MCsOnH9gZ0S9QHs12NWrNdApxsls1Vb5/NGrNdjcHx3dUas10UASoAjVEvhAbGagERgOpNkAPvxdbfKwaV2ikEj9SzXY1WPxABmDKQwdqzwRbM+12NSeJFGUEx3F8as10WwidLzVshDtxaPIbP13NdjVS9UZTYqgTVGrNcT9vyXU1'
+_cookie_uu = '3M3AXsquTU5Gur/Svik+ewflPm5Rk2ieY3BIPlLjyK3C0Dp9F8UoPgbTyKiGtZp4x1X+uAUGKD7BM2g+dVd8eqEzDErCoYvdcvGLvVLAen1y08hNQtALjVKAe+1hM8g9QbNonlG1/t4S82ieUsBbrSIQbq1yhV6tZ6ArvSbA7rgHc8n5AdReyAmDaJ5Wm/ee3VDoCnGj/LlBs2ieUZNorhHDKK5Q=='
+
 
 class IMDbURLopener(urllib.FancyURLopener):
     """Fetch web pages and handle errors."""
@@ -79,13 +87,32 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     urlOpener = IMDbURLopener()
 
+    def __init__(self, isThin=0, adultSearch=1, proxy=-1,
+                *arguments, **keywords):
+        """Initialize the access system."""
+        IMDbBase.__init__(self, *arguments, **keywords)
+        # When isThin is set, we're parsing the "maindetails" page
+        # of a movie (instead of the "combined" page) and movie/person
+        # references are not collected if no defaultModFunct is provided.
+        self.isThin = isThin
+        if isThin:
+            self.accessSystem = 'httpThin'
+            movie_parser.mdparse = 1
+            from utils import ParserBase
+            if self._defModFunct is None:
+                ParserBase.getRefs = 0
+                from imdb.utils import modNull
+                self._defModFunct = modNull
+        self.do_adult_search(adultSearch)
+        if proxy != -1: self.set_proxy(proxy)
+
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
-        return str(movieID).zfill(7)
+        return '%07d' % int(movieID)
 
     def _normalize_personID(self, personID):
         """Normalize the given personID."""
-        return str(personID).zfill(7)
+        return '%07d' % int(personID)
 
     def get_imdbMovieID(self, movieID):
         """Translate a movieID in an imdbID; in this implementation
@@ -119,6 +146,20 @@ class IMDbHTTPAccessSystem(IMDbBase):
         """Return the used proxy or an empty string."""
         return self.urlOpener.proxies.get('http', '')
 
+    def do_adult_search(self, doAdult,
+                        cookie_id=_cookie_id, cookie_uu=_cookie_uu):
+        """If doAdult is true, 'adult' movies are included in the
+        search results; cookie_id and cookie_uu are optional
+        parameters to select a specific account (see your cookie
+        or cookies.txt file."""
+        for index in xrange(len(self.urlOpener.addheaders)):
+            if self.urlOpener.addheaders[index][0] == 'Cookie':
+                del self.urlOpener.addheaders[index]
+                break
+        if doAdult:
+            c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
+            self.urlOpener.addheaders += [('Cookie', c_header)]
+
     def _retrieve(self, url):
         """Retrieve the given URL."""
         try:
@@ -135,12 +176,17 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def _search_movie(self, title, results):
         # The URL of the query.
-        params = urllib.urlencode({'more': 'tt', 'q': title})
+        # XXX: To retrieve the complete results list:
+        #      params = urllib.urlencode({'more': 'tt', 'q': title})
+        params = urllib.urlencode({'tt': 'on', 'mx': str(results), 'q': title})
         cont = self._retrieve(imdbURL_search % params)
-        return search_movie_parser.parse(cont)['data']
+        return search_movie_parser.parse(cont, results=results)['data']
 
     def get_movie_main(self, movieID):
-        cont = self._retrieve(imdbURL_movie % movieID + 'combined')
+        if not self.isThin:
+            cont = self._retrieve(imdbURL_movie % movieID + 'combined')
+        else:
+            cont = self._retrieve(imdbURL_movie % movieID + 'maindetails')
         return movie_parser.parse(cont)
 
     def get_movie_plot(self, movieID):
@@ -194,12 +240,42 @@ class IMDbHTTPAccessSystem(IMDbBase):
     def get_movie_connections(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'movieconnections')
         return connections_parser.parse(cont)
+
+    def get_movie_technical(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'technical')
+        return tech_parser.parse(cont)
+
+    def get_movie_business(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'business')
+        return tech_parser.parse(cont)
+
+    def get_movie_literature(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'literature')
+        return tech_parser.parse(cont)
+
+    def get_movie_locations(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'locations')
+        return locations_parser.parse(cont)
     
+    def get_movie_soundtrack(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'soundtrack')
+        return soundtrack_parser.parse(cont)
+
+    def get_movie_dvd(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'dvd')
+        return dvd_parser.parse(cont)
+
+    def get_movie_recommendations(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'recommendations')
+        return rec_parser.parse(cont)
+
     def _search_person(self, name, results):
         # The URL of the query.
-        params = urllib.urlencode({'more': 'nm', 'q': name})
+        # XXX: To retrieve the complete results list:
+        #      params = urllib.urlencode({'more': 'nm', 'q': name})
+        params = urllib.urlencode({'nm': 'on', 'mx': str(results), 'q': name})
         cont = self._retrieve(imdbURL_search % params)
-        return search_person_parser.parse(cont)['data']
+        return search_person_parser.parse(cont, results=results)['data']
 
     def get_person_main(self, personID):
         cont = self._retrieve(imdbURL_person % personID + 'maindetails')
@@ -222,6 +298,14 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_person % personID + 'otherworks')
         return otherworks_parser.parse(cont)
 
+    def get_person_agent(self, personID):
+        cont = self._retrieve(imdbURL_person % personID + 'agent')
+        return agent_parser.parse(cont)
+
+    def get_person_publicity(self, personID):
+        cont = self._retrieve(imdbURL_person % personID + 'publicity')
+        return publicity_parser.parse(cont)
+
     def get_person_official_sites(self, personID):
         cont = self._retrieve(imdbURL_person % personID + 'officialsites')
         return person_officialsites_parser.parse(cont)
