@@ -40,6 +40,10 @@ class HTMLMaindetailsParser(ParserBase):
         cparser = HTMLMaindetailsParser()
         result = cparser.parse(categorized_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _init(self):
         # This is the dictionary that will be returned by the parse() method.
         self.__person_data = {}
@@ -62,6 +66,9 @@ class HTMLMaindetailsParser(ParserBase):
         self.__in_b = 0
         self.__sect_name = ''
         self.__in_emailfriend = 0
+        self.__in_akas = 0
+        self.__aka = ''
+        self.__akas = []
 
     def get_data(self):
         """Return the dictionary."""
@@ -84,6 +91,8 @@ class HTMLMaindetailsParser(ParserBase):
                 self.__person_data['death date'] = d_date.strip()
             if d_notes:
                 self.__person_data['death notes'] = d_notes.strip()
+        if self.__akas:
+            self.__person_data['akas'] = self.__akas
         return self.__person_data
     
     def start_title(self, attrs):
@@ -111,6 +120,11 @@ class HTMLMaindetailsParser(ParserBase):
             self.__death += '::'
         elif self.__in_list:
             self.__in_list = 0
+        elif self.__in_akas:
+            self.__aka = self.__aka.strip()
+            if self.__aka:
+                self.__akas.append(self.__aka)
+                self.__aka = ''
 
     def start_b(self, attrs):
         self.__in_b = 1
@@ -143,7 +157,6 @@ class HTMLMaindetailsParser(ParserBase):
                     if d.has_key('name'):
                         self.__person_data['name'] = d['name']
 
-
     def start_ol(self, attrs): pass
 
     def end_ol(self):
@@ -204,6 +217,12 @@ class HTMLMaindetailsParser(ParserBase):
     def end_dd(self):
         self.__in_birth = 0
         self.__in_death = 0
+        if self.__in_akas:
+            self.__aka = self.__aka.strip()
+            if self.__aka:
+                self.__akas.append(self.__aka)
+                self.__aka = ''
+            self.__in_akas = 0
 
     def start_a(self, attrs):
         href = self.get_attr_value(attrs, 'href')
@@ -234,10 +253,14 @@ class HTMLMaindetailsParser(ParserBase):
             if self.__death and not self.__death[-1].isspace():
                 self.__death += ' '
             self.__death += sdata
+        elif self.__in_akas:
+            self.__aka += data
         elif sldata.startswith('date of death'):
             self.__in_death = 1
         elif sldata.startswith('date of birth'):
             self.__in_birth = 1
+        elif sldata.startswith('sometimes credited as'):
+            self.__in_akas = 1
         elif self.__in_sect:
             self.__sect_name += sldata
         elif self.__in_title:
@@ -299,10 +322,18 @@ class HTMLBioParser(ParserBase):
             # XXX: to get rid of the last colons.
             if sect[-1] == ':':
                 sect = sect[:-1]
+            if sect == 'salary': sect = 'salary history'
             data = self.__sect_data.strip()
             d_split = data.split('::')
+            d_split[:] = filter(None, [x.strip() for x in d_split])
+            if sect == 'salary history':
+                newdata = []
+                for j in d_split:
+                    j = filter(None, [x.strip() for x in j.split('@@@@')])
+                    newdata.append('::'.join(j))
+                d_split[:] = newdata
             if len(d_split) == 1:
-                self.__bio_data[sect] = data
+                self.__bio_data[sect] = d_split[0]
             # Multiple items are added separately (e.g.: 'trivia' is
             # a list of strings).
             else:
@@ -331,6 +362,14 @@ class HTMLBioParser(ParserBase):
                 self.__sect_data += '::'
 
     def end_tr(self): pass
+
+    def start_td(self, attrs): pass
+
+    def end_td(self):
+        if self.__in_sect and \
+                self.__sect_name.strip().lower() in \
+                ('salary', 'salary history'):
+            if self.__sect_data: self.__sect_data += '@@@@'
     
     def _handle_data(self, data):
         if self.__in_sect_name:
@@ -343,7 +382,7 @@ class HTMLBioParser(ParserBase):
 
 
 class HTMLOtherWorksParser(ParserBase):
-    """Parser for the "other works" page of a given person.
+    """Parser for the "other works" and "agent" pages of a given person.
     The page should be provided as a string, as taken from
     the akas.imdb.com server.  The final result will be a
     dictionary, with a key for every relevant section.
@@ -352,22 +391,34 @@ class HTMLOtherWorksParser(ParserBase):
         owparser = HTMLOtherWorksParser()
         result = owparser.parse(otherworks_html_string)
     """
+
+    def _init(self):
+        self.kind = 'other works'
+    
     def _reset(self):
         """Reset the parser."""
         self.__in_ow = 0
         self.__ow = []
         self.__cow = ''
+        self.__dostrip = 0
 
     def get_data(self):
         """Return the dictionary."""
         if not self.__ow: return {}
-        return {'other works': self.__ow}
+        return {self.kind: self.__ow}
 
     def start_dd(self, attrs):
         self.__in_ow = 1
 
     def end_dd(self): pass
 
+    def start_b(self, attrs): pass
+
+    def end_b(self):
+        if self.kind == 'agent' and self.__in_ow and self.__cow:
+            self.__cow += '::'
+            self.__dostrip = 1
+
     def do_br(self, attrs):
         if self.__in_ow and self.__cow:
             self.__ow.append(self.__cow.strip())
@@ -381,6 +432,9 @@ class HTMLOtherWorksParser(ParserBase):
     
     def _handle_data(self, data):
         if self.__in_ow:
+            if self.__dostrip:
+                data = data.lstrip()
+                if data: self.__dostrip = 0
             self.__cow += data
 
 
@@ -388,9 +442,14 @@ class HTMLOtherWorksParser(ParserBase):
 maindetails_parser = HTMLMaindetailsParser()
 bio_parser = HTMLBioParser()
 otherworks_parser = HTMLOtherWorksParser()
+agent_parser = HTMLOtherWorksParser()
+agent_parser.kind = 'agent'
 from movieParser import HTMLOfficialsitesParser
 person_officialsites_parser = HTMLOfficialsitesParser()
 from movieParser import HTMLAwardsParser
 person_awards_parser = HTMLAwardsParser()
 person_awards_parser.subject = 'name'
+from movieParser import HTMLTechParser
+publicity_parser = HTMLTechParser()
+publicity_parser.kind = 'publicity'
 
