@@ -24,18 +24,20 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import os
+from stat import ST_SIZE
 
 from imdb import IMDbBase
-from imdb._exceptions import IMDbDataAccessError, IMDbNotAvailable, IMDbError
+from imdb._exceptions import IMDbDataAccessError, IMDbError
 from imdb.utils import analyze_title, build_title, analyze_name, \
-                        build_name, re_titleRef, re_nameRef
+                        build_name, re_titleRef, re_nameRef, re_episodes
 from imdb.Person import Person
 from imdb.Movie import Movie
 
 from personParser import getFilmography, getBio, getAkaNames
 from movieParser import getLabel, getMovieCast, getAkaTitles, parseMinusList, \
                         getPlot, getRatingData, getMovieMisc, getTaglines, \
-                        getQuotes, getMovieLinks
+                        getQuotes, getMovieLinks, getBusiness, getLiterature, \
+                        getLaserdisc
 
 from ratober import search_name, search_title
 from utils import getFullIndex, sortBy, KeyFScan
@@ -68,6 +70,18 @@ class IMDbLocalAccessSystem(IMDbBase):
         self.__namesScan = KeyFScan('%snames.key' % self.__db)
         self.__titlesScan = KeyFScan('%stitles.key' % self.__db)
 
+    def _get_lastID(self, indexF):
+        fsize = os.stat(indexF)[ST_SIZE]
+        return (fsize / 4) - 1
+
+    def get_lastMovieID(self):
+        """Return the last movieID"""
+        return self._get_lastID('%stitles.index' % self.__db)
+    
+    def get_lastPersonID(self):
+        """Return the last personID"""
+        return self._get_lastID('%snames.index' % self.__db)
+
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
         try:
@@ -129,7 +143,7 @@ class IMDbLocalAccessSystem(IMDbBase):
         from imdb.parser.http.searchMovieParser import BasicMovieParser
         mparser = BasicMovieParser()
         result = mparser.parse(content)
-        if not (result and result.get('data')): return None
+        if not (result and result.has_key('data')): return None
         return result['data'][0][0]
 
     def get_imdbPersonID(self, personID):
@@ -146,7 +160,7 @@ class IMDbLocalAccessSystem(IMDbBase):
         from imdb.parser.http.searchPersonParser import BasicPersonParser
         pparser = BasicPersonParser()
         result = pparser.parse(content)
-        if not (result and result.get('data')): return None
+        if not (result and result.has_key('data')): return None
         return result['data'][0][0]
 
     def _findRefs(self, o, trefs, nrefs):
@@ -167,7 +181,6 @@ class IMDbLocalAccessSystem(IMDbBase):
                 rname = build_name(analyze_name(name, canonical=1),
                                     canonical=1)
                 if nrefs.has_key(rname): continue
-                nspl = name.split()
                 personID = self.__namesScan.getID(rname)
                 if personID is None:
                     personID = self.__namesScan.getID(name)
@@ -270,11 +283,10 @@ class IMDbLocalAccessSystem(IMDbBase):
             if data: res[name] = data
         if res.has_key('runtimes') and len(res['runtimes']) > 0:
             rt = res['runtimes'][0]
-            bi = rt.rfind('(')
-            ei = rt.rfind(')')
-            if bi != -1 and ei > bi:
-                res['runtimes'][0] = rt[:bi].strip()
-                res['notes'] = rt[bi:ei+1]
+            episodes = re_episodes.findall(rt)
+            if episodes:
+                res['runtimes'][0] = re_episodes.sub('', rt)
+                res['episodes'] = episodes[0]
         # AKA titles.
         akas = getAkaTitles(movieID,
                     '%saka-titles.data' % self.__db,
@@ -293,10 +305,6 @@ class IMDbLocalAccessSystem(IMDbBase):
                         'titlesRefs': trefs, 'namesRefs': nrefs}
         return {'data': {}}
 
-    def get_movie_awards(self, movieID):
-        raise IMDbNotAvailable, 'awards information not available ' + \
-                                'in the local database'
-
     def get_movie_taglines(self, movieID):
         tg = getTaglines(movieID, '%staglines.index' % self.__db,
                         '%staglines.data' % self.__db)
@@ -336,6 +344,14 @@ class IMDbLocalAccessSystem(IMDbBase):
         if goo: return {'data': {'goofs': goo},
                         'titlesRefs': trefs, 'namesRefs': nrefs}
         return {'data': {}}
+    
+    def get_movie_soundtrack(self, movieID):
+        goo = parseMinusList(movieID, '%ssoundtracks.data' % self.__db,
+                            '%ssoundtracks.index' % self.__db)
+        trefs, nrefs = self._extractRefs(goo)
+        if goo: return {'data': {'soundtrack': goo},
+                        'titlesRefs': trefs, 'namesRefs': nrefs}
+        return {'data': {}}
 
     def get_movie_quotes(self, movieID):
         mq = getQuotes(movieID, '%squotes.data' % self.__db,
@@ -359,10 +375,6 @@ class IMDbLocalAccessSystem(IMDbBase):
         data = getRatingData(movieID, '%sratings.data' % self.__db)
         return {'data': data}
 
-    def get_movie_official_sites(self, movieID):
-        raise IMDbNotAvailable, 'official sites information not available ' + \
-                                'in the local database'
-
     def get_movie_trivia(self, movieID):
         triv = parseMinusList(movieID, '%strivia.data' % self.__db,
                             '%strivia.index' % self.__db)
@@ -387,7 +399,31 @@ class IMDbLocalAccessSystem(IMDbBase):
                             '%stitles.key' % self.__db)
         if mc: return {'data': {'connections': mc}}
         return {'data': {}}
+
+    def get_movie_business(self, movieID):
+        mb = getBusiness(movieID, '%sbusiness.index' % self.__db,
+                            '%sbusiness.data' % self.__db)
+        trefs, nrefs = self._extractRefs(mb)
+        if mb: return {'data': {'business': mb},
+                        'titlesRefs': trefs, 'namesRefs': nrefs}
+        return {'data': {}}
     
+    def get_movie_literature(self, movieID):
+        ml = getLiterature(movieID, '%sliterature.index' % self.__db,
+                            '%sliterature.data' % self.__db)
+        trefs, nrefs = self._extractRefs(ml)
+        if ml: return {'data': {'literature': ml},
+                        'titlesRefs': trefs, 'namesRefs': nrefs}
+        return {'data': {}}
+    
+    def get_movie_laserdisc(self, movieID):
+        ml = getLaserdisc(movieID, '%slaserdisc.index' % self.__db,
+                            '%slaserdisc.data' % self.__db)
+        trefs, nrefs = self._extractRefs(ml)
+        if ml: return {'data': {'laserdisc': ml},
+                        'titlesRefs': trefs, 'namesRefs': nrefs}
+        return {'data': {}}
+
     def _search_person(self, name, results):
         # ratober functions return a sorted
         # list of tuples (match_score, personID, personName)
@@ -450,15 +486,7 @@ class IMDbLocalAccessSystem(IMDbBase):
     def get_person_biography(self, personID):
         return self.get_person_main(personID)
 
-    def get_person_awards(self, personID):
-        raise IMDbNotAvailable, 'awards information not available ' + \
-                                'in the local database'
-
     def get_person_other_works(self, personID):
         return self.get_person_main(personID)
 
-    def get_person_official_sites(self, personID):
-        raise IMDbNotAvailable, 'official sites information not available ' + \
-                                'in the local database'
-
 
